from typing import Dict
from dataclasses import dataclass, field

@dataclass
class AndromedaPrompt:
    name: str
    prompt_template: str
    input_vars: Dict[str, str]
    output_vars: Dict[str, str]
    guidance_kwargs: Dict[str, str] = field(default_factory=dict)
    macro_vars: Dict[str, str] = field(default_factory=dict)



from andromeda_chain.prompt import AndromedaPrompt
from andromeda_chain.response import AndromedaResponse

import requests
from copy import deepcopy
from typing import Dict, Optional


class AndromedaChain:
    def __init__(self, guidance_url: str = "http://0.0.0.0:9000") -> None:
        self._guidance_url = guidance_url


    def run_guidance_prompt(self, guidance_prompt: AndromedaPrompt, input_vars: Dict[str, str], macro_values:Optional[dict]=None) -> AndromedaResponse:
        """Interprets the guidance prompt using a guidance server
        
        Also expands macro calls.

        Parameters: 
            guidance_prompt (GuidancePrompt): The prompt template to use for the guidance.
            input_vars (Dict[str, str]): The input variables to use for the guidance.
            macro_values (Optional[dict]): Macro values to be expanded. Not all prompts need this.

        Returns: 
            GuidanceResponse: The response from the guidance server

        """
        # Macro variables are generaly currently not necessary
        # Unless you want to dynamically set guidance settings that are normally not settable.
        # One example is the number of iterations guidance should loop on a specific generation. 

        macro_vars = deepcopy(guidance_prompt.macro_vars)
        prompt_str = guidance_prompt.prompt_template
        if macro_values is not None:
            keys = list(macro_vars.keys())
            for key in keys:
                macro_identifier = macro_vars.pop(key)
                value = macro_values.pop(key)
                prompt_str = prompt_str.replace(macro_identifier, str(value))
        
        if macro_vars:
            raise TypeError("Unexpanded macro variables in prompt! Fix your prompt or pass the macro variable")

        result = self._call_guidance(
            prompt_template=prompt_str,
            input_vars=input_vars,
            output_vars=guidance_prompt.output_vars,
            guidance_kwargs=guidance_prompt.guidance_kwargs
        )
        expanded_generation = result.pop("__main__")
        return AndromedaResponse(
            expanded_generation=expanded_generation,
            result_vars=result,
        )


    def _call_guidance(self, prompt_template, output_vars, input_vars=None, guidance_kwargs=None):
        """
        This function calls a guidance API with the given parameters and returns the response.
        
        Parameters:
        prompt_template (str): The prompt template to use for the guidance.
        output_vars (dict): The output variables to use for the guidance.
        input_vars (dict): The input variables to use for the guidance.
        guidance_kwargs (dict): The guidance keywords to use for the guidance.
        
        Returns:
        dict: The response from the guidance API.
        """
        
        if input_vars is None:
            input_vars = {}
        if guidance_kwargs is None:
            guidance_kwargs = {}
        
        data = {
            "prompt_template": prompt_template,
            "output_vars": output_vars,
            "guidance_kwargs": guidance_kwargs,
            "input_vars": input_vars,
        }
        
        response = requests.post(
            self._guidance_url,
            json=data
        )
        
        response.raise_for_status()
        
        return response.json()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import codecs
from setuptools import setup, find_packages
from distutils.util import convert_path

def read(fname):
    file_path = os.path.join(os.path.dirname(__file__), fname)
    return codecs.open(file_path, encoding="utf-8").read()

main_ns = {}
ver_path = convert_path('src/andromeda_chain/version.py')
with open(ver_path) as ver_file:
    exec(ver_file.read(), main_ns)

setup(
    name="andromeda-chain",
    version=main_ns["__version__"],
    author="Paolo Rechia",
    author_email="paolorechia@gmail.com",
    maintainer="Paolo Rechia",
    maintainer_email="paolorechia@gmail.com",
    license="MIT",
    url="https://github.com/ChuloAI/andromeda-chain",
    description="A client to query a HTTP Guidance server",
    python_requires=">=3.9",
    install_requires=["requests"],
    package_dir={'':'src'},
    packages=find_packages("src", exclude=["guidance_server"]),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: Implementation :: CPython",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: MIT License",
    ]
)